# Example b211u
# 2/3 heart mesh in RC coordinates - 520K grid points
#  with a Noble 98 ionic current model - monodomain -
#  implicit solution with Adams Moulton integrator.
#
# Output: dipoles.ipsour - dipole source file for use
#          inside passive torso
#         activ.exelem - activation times from each
#          collocation point
#
# WARNING - takes approx 4 days to solve on HPC1 with
#  8 threads.

# Geometry file string
$GEOMSTRING = "translated";

# Stimulus sites
$ACTIV1 = "307003,416419,409772,307643,307644,307659,308283";
$ACTIV2 = "316265,425989,317184,317185,340563,317208,318105";
$ACTIV3 = " 36057,192817, 37376, 37377, 37378, 37410, 38697";

#=================================================

{
  eval { require Time::HiRes };
  if( $@ )
    {
      print "warning: $@";
    }
  else
    {
      import Time::HiRes qw(time);
    }
}

sub cpu_time()
  {
    # Add user/system parent/child cpu_times.
    my $result = 0;
    map { $result += $_ } times;
    $result;
  }

sub print_timing($$$)
  {
    my ($message,$wall,$cpu) = @_;
    printf "%s: %.3fs wall, %.3fs cpu.\n", $message, $wall, $cpu;
  }

sub print_memory_use()
  {
    # Different operating systems have different data and ps uses different
    # arguments so try a series of commands in an attempt to collect the
    # information.
    system qw(ps -o vsz -p), $$;
    system qw(ps -l -p), $$;
    system qw(ps v), $$;
  }

set echo;

if (!defined $numThreads) {
    $numThreads = 4;
}
set num_threads $numThreads;

if (!defined $input) {
    $input = "input";
}

if (!defined $output) {
    $output = sprintf "output-%03d",$numThreads;
}

if (! -d $output) {
    mkdir $output,0755;
}

# Read in the 2/3 rc heart mesh
$file = "$input/$GEOMSTRING";
FEM define para;r;$file;
FEM define coor 3,1;
FEM define node;r;$file;
FEM define base;r;$file;
FEM define elem;r;$file;
FEM define fibr;r;$file;
FEM define elem;r;$file fibre;

# Create a collocation mesh
FEM define grid;r;$input/full_res;

FEM update grid geometry;
FEM update grid metric;
FEM group grid external as boundary;
FEM group grid grid $ACTIV1 as stimulus1;
FEM group grid grid $ACTIV2 as stimulus2;
FEM group grid grid $ACTIV3 as stimulus3;

# Set up the problem ready for solution
FEM define equa;r;$input/mono;
FEM define mate;r;"$input/621";
FEM define cell;r;$input/n98;
FEM define mate;r;$input/n98 cell;
FEM update grid material;
FEM define init;r;$input/n98;
FEM define solv;r;$input/implicit_05;
FEM solve to 0 calc_activ;

($wall_setup,$cpu_setup) = (time, cpu_time);
print_timing 'setup', $wall_setup - $^T, $cpu_setup;

#Export the geometry of the mesh
FEM export node;$output/heart as heart;
FEM export elem;$output/heart as heart;
($wall_output,$cpu_output) = (time, cpu_time);
print_timing 'initial output', $wall_output - $wall_setup, $cpu_output - $cpu_setup;

if (!defined $tEnd) {
    $tEnd = 10;
}
if (!defined $dT) {
    $dT = 1;
}
for( my $time = 0; $time < $tEnd; $time+=$dT ) {
    FEM solve restart to $time calc_activ;
    ($wall_solve,$cpu_solve) = (time, cpu_time);
    print_timing "solve $time", $wall_solve - $wall_output, $cpu_solve - $cpu_output;
    FEM define sour;c grid grregion 1 grclass 1 time $time fixed;
    my $file = sprintf "$output/potential_%05d",$time;
    fem export element;$file field as heart;
    ($wall_output,$cpu_output) = (time, cpu_time);
    print_timing "output and source$time", $wall_output - $wall_solve, $cpu_output - $cpu_solve;
}

($wall_solve,$cpu_solve) = (time, cpu_time);
print_timing 'total solve', $wall_solve - $wall_setup, $cpu_solve - $cpu_setup;

FEM update grid activation_times;
FEM export elem;$output/activ as heart iy 4 field;
FEM define sour;w;$output/dipoles grid grregion 1 grclass 1;

print_memory_use;
